/*
 Copyright 2020 Appboxo pte. ltd.
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at
 
 http://www.apache.org/licenses/LICENSE-2.0
 
 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 */

import Flutter
import UIKit
import AppBoxoSDK


public class SwiftAppboxoSdkPlugin: NSObject, FlutterPlugin {
    
    private let paymentEventsStreamHandler = PaymentEventsStreamHandler()
    private let customEventsStreamHandler = CustomEventsStreamHandler()
    private let lifecycleEventsStreamHandler = LifecycleEventsStreamHandler()
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let instance = SwiftAppboxoSdkPlugin()
        
        let channel = FlutterMethodChannel(name: "appboxo_sdk", binaryMessenger: registrar.messenger())
        registrar.addMethodCallDelegate(instance, channel: channel)
        
        let paymentEventChannel = FlutterEventChannel(name: "appboxo_sdk_payment_events", binaryMessenger: registrar.messenger())
        let customEventChannel = FlutterEventChannel(name: "appboxo_sdk_custom_events", binaryMessenger: registrar.messenger())
        let lifecycleChannel = FlutterEventChannel(name: "appboxo_sdk_lifecycle", binaryMessenger: registrar.messenger())
        
        paymentEventChannel.setStreamHandler(instance.paymentEventsStreamHandler)
        customEventChannel.setStreamHandler(instance.customEventsStreamHandler)
        lifecycleChannel.setStreamHandler(instance.lifecycleEventsStreamHandler)
    }
    
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        
        switch call.method {
        case "setConfig":
            guard let arguments = call.arguments as? [String : Any] else { return }
            let clientId = arguments["client_id"] as? String ?? ""
            let sandboxMode = arguments["sandbox_mode"] as? Bool ?? false
            let theme = arguments["theme"] as? String ?? ""
            let showPermissionsPage = arguments["show_permissions_page"] as? Bool ?? true
            let showClearCache = arguments["show_clear_cache"] as? Bool ?? true
            
            var globalTheme : Theme = .System
            switch (theme) {
            case "dark":
                globalTheme = .Dark
            case "light":
                globalTheme = .Light
            default:
                globalTheme = .System
            }
            
            let config = Config(clientId: clientId, theme: globalTheme)
            config.sandboxMode = sandboxMode
            config.permissionsPage = showPermissionsPage
            config.showClearCache = showClearCache
            
            Appboxo.shared.setConfig(config: config)
            
        case "openMiniapp":
            guard let arguments = call.arguments as? [String : Any] else { return }
            let appId = arguments["app_id"] as? String ?? ""
            let data = arguments["data"] as? [String: Any] ?? nil
            let theme = arguments["theme"] as? String ?? nil
            let extraUrlParams = arguments["extra_url_params"] as? [String: String] ?? nil
            let colors = arguments["colors"] as? [String: String] ?? nil
            
            let miniApp = Appboxo.shared.getMiniapp(appId: appId)
            miniApp.setData(data: data)
            miniApp.delegate = self
            
            let miniappConfig = MiniappConfig()
            miniappConfig.setExtraParams(extraParams: extraUrlParams)
            
            if let theme = theme {
                switch theme {
                case "dark":
                    miniappConfig.setTheme(theme: .Dark)
                case "light":
                    miniappConfig.setTheme(theme: .Light)
                case "system":
                    miniappConfig.setTheme(theme: .System)
                default:
                    break
                }
            }
            
            if let colors = colors {
                miniappConfig.setColor(color: MiniappColor(primary: colors["primary_color"] ?? "", secondary: colors["secondary_color"] ?? "", tertiary: colors["tertiary_color"] ?? ""))
            }
            
            miniApp.setConfig(config: miniappConfig)
            miniApp.open(viewController: UIApplication.shared.delegate!.window!!.rootViewController!)
            
        case "closeMiniapp":
            guard let arguments = call.arguments as? [String : Any] else { return }
            let appId = arguments["app_id"] as? String ?? ""
            if let miniapp = Appboxo.shared.getExistingMiniapp(appId: appId) {
                miniapp.close()
            }
            
        case "sendEvent":
            guard let arguments = call.arguments as? [String : Any] else { return }
            let map = arguments["custom_event"] as? [String: Any] ?? [:]
            let appId = map["app_id"] as? String ?? ""
            
            let customEvent = CustomEvent()
            customEvent.requestId = map["request_id"] as? Int ?? 0
            customEvent.type = map["type"] as? String ?? ""
            customEvent.errorType = map["error_type"] as? String ?? ""
            customEvent.payload = map["payload"] as? [String : Any] ?? nil
            
            Appboxo.shared.getMiniapp(appId: appId).sendCustomEvent(customEvent: customEvent)
            
        case "sendPaymentEvent":
            guard let arguments = call.arguments as? [String : Any] else { return }
            let map = arguments["payment_event"] as? [String: Any] ?? [:]
            let appId = map["app_id"] as? String ?? ""
            
            let paymentData = PaymentData()
            paymentData.transactionToken = map["transaction_token"] as? String ?? ""
            paymentData.miniappOrderId = map["miniapp_order_id"] as? String ?? ""
            paymentData.amount = map["amount"] as? Double ?? 0
            paymentData.currency = map["currency"] as? String ?? ""
            paymentData.status = map["status"] as? String ?? ""
            paymentData.hostappOrderId = map["hostapp_order_id"] as? String ?? ""
            paymentData.extraParams = map["extra_params"] as? [String : Any] ?? nil
            
            Appboxo.shared.getMiniapp(appId: appId).sendPaymentEvent(paymentData: paymentData)
            
        case "setAuthCode":
            guard let arguments = call.arguments as? [String : Any] else { return }
            let appId = arguments["app_id"] as? String ?? ""
            let authCode = arguments["auth_code"] as? String ?? ""
            
            Appboxo.shared.getMiniapp(appId: appId).setAuthCode(authCode: authCode)
            
        case "hideAllMiniapps":
            Appboxo.shared.hideMiniapps()
            
        case "logout":
            Appboxo.shared.logout()
            
        default:
            break
        }
    }
}

extension SwiftAppboxoSdkPlugin : MiniappDelegate {
    public func didReceivePaymentEvent(miniapp: Miniapp, paymentData: PaymentData) {
        let dict = [
            "app_id" : miniapp.appId,
            "transaction_token" : paymentData.transactionToken,
            "miniapp_order_id" : paymentData.miniappOrderId,
            "amount" : paymentData.amount,
            "currency" : paymentData.currency,
            "status" : paymentData.status,
            "hostapp_order_id" : paymentData.hostappOrderId,
            "extra_params" : paymentData.extraParams ?? [String : Any]()
        ] as [String: Any]
        
        paymentEventsStreamHandler.events?(dict)
    }
    
    public func didReceiveCustomEvent(miniapp: Miniapp, customEvent: CustomEvent) {
        let dict = [
            "app_id" : miniapp.appId,
            "request_id" : customEvent.requestId,
            "type" : customEvent.type,
            "error_type" : customEvent.errorType,
            "payload" : customEvent.payload ?? [String : Any]()
        ] as [String: Any]
        
        customEventsStreamHandler.events?(dict)
    }
    
    public func onLaunch(miniapp: Miniapp) {
        let dict = [
            "app_id" : miniapp.appId,
            "lifecycle" : "onLaunch"
        ]
        
        lifecycleEventsStreamHandler.events?(dict)
    }
    
    public func onResume(miniapp: Miniapp) {
        let dict = [
            "app_id" : miniapp.appId,
            "lifecycle" : "onResume"
        ]
        
        lifecycleEventsStreamHandler.events?(dict)
    }
    
    public func onPause(miniapp: Miniapp) {
        let dict = [
            "app_id" : miniapp.appId,
            "lifecycle" : "onPause"
        ]
        
        lifecycleEventsStreamHandler.events?(dict)
    }
    
    public func onClose(miniapp: Miniapp) {
        let dict = [
            "app_id" : miniapp.appId,
            "lifecycle" : "onClose"
        ]
        
        lifecycleEventsStreamHandler.events?(dict)
    }
    
    public func onError(miniapp: Miniapp, message: String) {
        let dict = [
            "app_id" : miniapp.appId,
            "lifecycle" : "onError",
            "error" : message
        ]
        
        lifecycleEventsStreamHandler.events?(dict)
    }
    
    public func onUserInteraction(miniapp: Miniapp) {
        let dict = [
            "app_id" : miniapp.appId,
            "lifecycle" : "onUserInteraction"
        ]
        
        lifecycleEventsStreamHandler.events?(dict)
    }
    
    public func onAuth(miniapp: Miniapp) {
        let dict = [
            "app_id" : miniapp.appId,
            "lifecycle" : "onAuth"
        ]
        
        lifecycleEventsStreamHandler.events?(dict)
    }
}
