/*
Copyright 2020 Appboxo pte. ltd.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

import 'dart:async';
import 'dart:ffi';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class Appboxo {
  static const MethodChannel _channel = const MethodChannel('appboxo_sdk');
  static const EventChannel _customEventChannel =
      const EventChannel('appboxo_sdk_custom_events');
  static const EventChannel _paymentEventChannel =
      const EventChannel('appboxo_sdk_payment_events');
  static const EventChannel _lifecycleEventChannel =
      const EventChannel('appboxo_sdk_lifecycle');

  static void setConfig(String clientId,
      {bool? sandboxMode = false,
      bool? multitaskMode = true,
      String? theme,
      bool? isDebug = false,
      bool? showPermissionsPage = true,
      bool? showClearCache = true}) {
    _channel.invokeMethod("setConfig", {
      'client_id': clientId,
      'sandbox_mode': sandboxMode,
      'multitask_mode': multitaskMode,
      'theme': theme == 'dark' || theme == 'light' ? theme : 'system',
      'is_debug': isDebug,
      'show_permissions_page': showPermissionsPage,
      'show_clear_cache': showClearCache
    });
  }

  static void openMiniapp(String appId,
      {Map<String, dynamic>? data,
      String? theme,
      Map<String, String>? extraUrlParams,
      String? urlSuffix,
      Map<String, String>? colors}) {
    _channel.invokeMethod("openMiniapp", {
      'app_id': appId,
      'data': data,
      'theme': theme == 'dark' || theme == 'light' || theme == 'system'
          ? theme
          : null,
      'extra_url_params': extraUrlParams,
      'url_suffix': urlSuffix,
      'colors': {
        "primary_color": colors?["primary_color"],
        "secondary_color": colors?["secondary_color"],
        "tertiary_color": colors?["tertiary_color"]
      }
    });
  }

  static void closeMiniapp(String appId) {
    _channel.invokeMethod("closeMiniapp", {'app_id': appId});
  }

  static void setAuthCode(String appId, String authCode) {
    _channel
        .invokeMethod("setAuthCode", {'app_id': appId, 'auth_code': authCode});
  }

  static void hideMiniapps() {
    _channel.invokeMethod("hideAllMiniapps");
  }

  static Stream<CustomEvent> customEvents() {
    return _customEventChannel
        .receiveBroadcastStream()
        .map((dynamic event) => CustomEvent.fromJson(event));
  }

  static Stream<PaymentEvent> paymentEvents() {
    return _paymentEventChannel
        .receiveBroadcastStream()
        .map((dynamic event) => PaymentEvent.fromJson(event));
  }

  static Future<void> Function() lifecycleHooksListener(
      {Function(String appId)? onAuth,
      Function(String appId)? onLaunch,
      Function(String appId)? onResume,
      Function(String appId)? onPause,
      Function(String appId)? onClose,
      Function(String appId, String? error)? onError,
      Function(String appId)? onUserInteraction}) {
    StreamSubscription<LifecycleHook> subscription = _lifecycleEventChannel
        .receiveBroadcastStream()
        .map((event) => LifecycleHook.fromJson(event))
        .listen((hook) {
      switch (hook.lifecycle) {
        case MiniappLifecycleHooks.OnAuth:
          onAuth?.call(hook.appId);
          break;
        case MiniappLifecycleHooks.OnLaunch:
          onLaunch?.call(hook.appId);
          break;
        case MiniappLifecycleHooks.OnResume:
          onResume?.call(hook.appId);
          break;
        case MiniappLifecycleHooks.OnPause:
          onPause?.call(hook.appId);
          break;
        case MiniappLifecycleHooks.OnClose:
          onClose?.call(hook.appId);
          break;
        case MiniappLifecycleHooks.OnError:
          onError?.call(hook.appId, hook.error);
          break;
        case MiniappLifecycleHooks.OnUserInteraction:
          onUserInteraction?.call(hook.appId);
          break;
      }
    });

    return () => subscription.cancel();
  }

  static void sendEvent(CustomEvent customEvent) {
    _channel.invokeMethod("sendEvent", {'custom_event': customEvent.toJson()});
  }

  static void sendPaymentEvent(PaymentEvent paymentEvent) {
    _channel.invokeMethod(
        "sendPaymentEvent", {'payment_event': paymentEvent.toJson()});
  }

  static void logout() {
    _channel.invokeMethod("logout");
  }
}

class LifecycleHook {
  String appId;
  String? lifecycle;
  String? error;

  LifecycleHook(this.appId, {this.lifecycle, this.error});

  LifecycleHook.fromJson(Map<dynamic, dynamic> json)
      : appId = json['app_id'],
        lifecycle = json['lifecycle'],
        error = json['error'];

  Map<String, dynamic> toJson() => {
        'app_id': appId,
        'lifecycle': lifecycle,
        'error': error,
      };
}

class CustomEvent {
  String appId;
  int? requestId;
  String type;
  String? errorType;
  Map payload;

  CustomEvent(this.type, this.appId, this.payload, this.requestId,
      {this.errorType});

  CustomEvent.fromJson(Map<dynamic, dynamic> json)
      : appId = json['app_id'],
        requestId = json['request_id'],
        type = json['type'],
        errorType = json['error_type'],
        payload = json['payload'];

  Map<String, dynamic> toJson() => {
        'app_id': appId,
        'request_id': requestId,
        'type': type,
        'error_type': errorType,
        'payload': payload
      };
}

class PaymentEvent {
  String appId;
  String transactionToken;
  String miniappOrderId;
  double amount;
  String currency;
  String status;
  String hostappOrderId;
  Map? extraParams;

  PaymentEvent(this.appId, this.transactionToken, this.miniappOrderId,
      this.amount, this.currency, this.status, this.hostappOrderId,
      {this.extraParams});

  PaymentEvent.fromJson(Map<dynamic, dynamic> json)
      : appId = json['app_id'],
        transactionToken = json['transaction_token'],
        miniappOrderId = json['miniapp_order_id'],
        amount = json['amount'],
        currency = json['currency'],
        status = json['status'],
        hostappOrderId = json['hostapp_order_id'],
        extraParams = json['extra_params'];

  Map<String, dynamic> toJson() => {
        'app_id': appId,
        'transaction_token': transactionToken,
        'miniapp_order_id': miniappOrderId,
        'amount': amount,
        'currency': currency,
        'status': status,
        'hostapp_order_id': hostappOrderId,
        'extra_params': extraParams
      };
}

class MiniappLifecycleHooks {
  static const OnAuth = 'onAuth';
  static const OnLaunch = 'onLaunch';
  static const OnResume = 'onResume';
  static const OnPause = 'onPause';
  static const OnClose = 'onClose';
  static const OnError = 'onError';
  static const OnUserInteraction = 'onUserInteraction';
}
